@@ -38,10 +38,10 @@
 #' dlq <- distLquantile(annMax, weight=FALSE, quiet=TRUE, probs=0.97, returnlist=TRUE)
 #' dlq$quant
 #' distLplot(dlq, qlines=TRUE) # per default best fitting distribution functions
-#' distLplot(dlq, qlines=TRUE, qrow=c("wak","q_gpd*"), nbest=14)
+#' distLplot(dlq, qlines=TRUE, qrow=c("wak","GPD*"), nbest=14)
 #' #pdf("dummy.pdf", width=9)
-#' distLplot(dlq, qlines=TRUE, qrow="q_gpd*", nbest=13, xlim=c(102,110), 
-#'           qlinargs=list(lwd=3), qheights=seq(0.02, 0.005, len=13))
+#' distLplot(dlq, qlines=TRUE, qrow="GPD*", nbest=13, xlim=c(102,110), 
+#'           qlinargs=list(lwd=3), qheights=seq(0.02, 0.005, len=14))
 #' #dev.off()
 #'
 #'
@@ -89,6 +89,12 @@
 #' library("berryFunctions")
 #' textField(rep(0.5,4), trunceffect[,11], paste0("Q",myprobs*100,"%") )
 #' par(op)
+#' 
+#' trunc <- seq(0,0.1,len=200)
+#' dd <- pbsapply(trunc, function(t) distLquantile(annMax, 
+#'           selection="gpa", weight=FALSE, truncate=t, prob=0.99, quiet=T)[c(1,3),])
+#   plot(trunc, dd[1,], type="o", las=1)
+#' lines(trunc, dd[2,], type="o", col=2)
 #'
 #'
 #' set.seed(3); rnum <- rlmomco(n=1e3, para=dlf$parameter$gpa)
@@ -108,25 +114,49 @@
 #' 
 #' } # end dontrun
 #' 
-#' @param x Sample for which parametrical quantiles are to be calculated. If it is NULL (the default), \code{dat} from \code{dlf} is used. DEFAULT: NULL
+#' @param x Sample for which parametrical quantiles are to be calculated. 
+#'          If it is NULL (the default), \code{dat} from \code{dlf} is used. DEFAULT: NULL
 #' @param probs Numeric vector of probabilities with values in [0,1]. DEFAULT: c(0.8,0.9,0.99)
-#' @param truncate Number between 0 and 1 (proportion of sample discarded). Censored quantile: fit to highest values only (truncate lower proportion of x). Probabilities are adjusted accordingly. DEFAULT: 0
-#' @param threshold POT cutoff value. If you want correct percentiles, set this only via truncate, see Details of \code{\link{q_gpd}}. DEFAULT: \code{\link[berryFunctions]{quantileMean}(x, truncate)}
-#' @param selection Distribution type, eg. "gev" or "wak", see \code{\link[lmomco]{dist.list} in lmomco}. Can be a vector. If NULL (the default), all types present in dlf$parameter are used. DEFAULT: NULL
-#' @param dlf dlf object described in \code{\link{extremeStat}}. Use this to save computing time for large datasets where you already have dlf. DEFAULT: NULL
-#' @param order Sort results by GOF? If TRUE (the default) and length(selection)>1, the output is ordered by dlf$gof, else by order of appearance in selection (or dlf$parameter). DEFAULT: TRUE
-#' @param returnlist Return full \code{dlf}list with output attached as element \code{quant}? If FALSE (the default), just the matrix with quantile estimates is returned. DEFAULT: FALSE
-#' @param empirical Add empirical \code{\link{quantileMean}} in the output matrix and vertical lines? DEFAULT: TRUE
-#' @param weighted Include weighted averages across distribution functions to the output? DEFAULT: empirical, so additional options can all be excluded with emp=F.
-#' @param gpd Include GPD quantile estimation via \code{\link{q_gpd}}? DEFAULT: empirical
-#' @param addinfo Should information like sample size be \code{\link{rbind}ed} to the output? DEFAULT: FALSE
-#' @param speed Compute \code{\link{q_gpd}} only for fast methods? Don't accidentally set this to \code{FALSE} in simulations or with large datasets! DEFAULT: TRUE
+#' @param truncate Number between 0 and 1 (proportion of sample discarded). 
+#'                 Censored quantile: fit to highest values only (truncate lower proportion of x). 
+#'                 Probabilities are adjusted accordingly. DEFAULT: 0
+#' @param threshold POT cutoff value. If you want correct percentiles, 
+#'                  set this only via truncate, see Details of \code{\link{q_gpd}}. 
+#'                  DEFAULT: \code{\link[berryFunctions]{quantileMean}(x, truncate)}
+#' @param selection Distribution type, eg. "gev" or "wak", see \code{\link[lmomco]{dist.list} in lmomco}. 
+#'                  Can be a vector. If NULL (the default), all types present in 
+#'                  dlf$parameter are used. DEFAULT: NULL
+#' @param dlf dlf object described in \code{\link{extremeStat}}. Use this to save 
+#'            computing time for large datasets where you already have dlf. DEFAULT: NULL
+#' @param order Sort results by GOF? If TRUE (the default) and length(selection)>1, 
+#'              the output is ordered by dlf$gof, else by order of appearance in 
+#'              selection (or dlf$parameter). DEFAULT: TRUE
+#' @param returnlist Return full \code{dlf}list with output attached as element \code{quant}? 
+#'                   If FALSE (the default), just the matrix with quantile estimates 
+#'                   is returned. DEFAULT: FALSE
+#' @param empirical Add empirical \code{\link{quantileMean}} in the output matrix 
+#'                  and vertical lines? DEFAULT: TRUE
+#' @param weighted Include weighted averages across distribution functions to the output?
+#'                 DEFAULT: empirical, so additional options can all be excluded with emp=F.
+#' @param gpd Include GPD quantile estimation via \code{\link{q_gpd}}? 
+#'            Note that the 'GPD_LMO_lmomco' result differs slightly from 'gpa' if
+#'            truncate=0. This comes from using x>threshold ('GPD_*') or
+#'            x>=threshold ('gpa' and all other distributions in extremeStat). 
+#'            DEFAULT: empirical
+#' @param addinfo Should information about sample size and threshold be 
+#'                \code{\link{rbind}ed} to the output? DEFAULT: FALSE
+#' @param speed Compute \code{\link{q_gpd}} only for fast methods? 
+#'              Don't accidentally set this to \code{FALSE} in simulations or 
+#'              with large datasets! DEFAULT: TRUE
 #' @param plot Should \code{\link{distLplot}} be called? DEFAULT: FALSE
-#' @param plotargs List of arguments to be passed to \code{\link{distLplot}} like qlines, qheights, qrow, qlinargs, nbest, cdf, ...
+#' @param plotargs List of arguments to be passed to \code{\link{distLplot}} 
+#'                 like qlines, qheights, qrow, qlinargs, nbest, cdf, ...
 #' @param quiet Suppress notes? DEFAULT: FALSE
 #' @param ssquiet Suppress sample size notes? DEFAULT: quiet
-#' @param ttquiet Suppress truncation!=threshold note? Note that \code{\link{q_gpd}} is called with ttquiet=TRUE. DEFAULT: quiet
-#' @param \dots Arguments passed to \code{\link{distLfit}} (and \code{\link{distLplot}} if plot=TRUE).
+#' @param ttquiet Suppress truncation!=threshold note? Note that \code{\link{q_gpd}} 
+#'                is called with ttquiet=TRUE. DEFAULT: quiet
+#' @param \dots Arguments passed to \code{\link{distLfit}}
+#'              (and \code{\link{distLplot}} if plot=TRUE).
 #'
 distLquantile <- function(
 x=NULL,
@@ -167,7 +197,7 @@ if(!is.null(dlf)) if(any(dlf$truncate!=truncate)|any(any(dlf$threshold!=threshol
   currentdlfthreshold <- dlf$threshold
   if(!quiet) on.exit(message("Note in distLquantile: truncate (",truncate,
        ") did not match dlf$truncate (",currentdlftruncate,
-       "). Thresholds: ",pastec(signif(c(threshold, currentdlfthreshold),7)),
+       "). Thresholds: ",toString(signif(c(threshold, currentdlfthreshold),7)),
        ".\n   distLfit is called with the original dlf$dat."), add=TRUE)
   x <- dlf$dat
   internaldatname <- dlf$datname
@@ -197,9 +227,9 @@ if(!is.null(selection))
 # Empty output matrix: ---------------------------------------------------------
 dn <- names(dlf$parameter)  # dn = distribution names
 dn2 <- rownames(dlf$gof)
-if(any(!dn2 %in% dn)) warning(pastec(dn2[!dn2 %in% dn]),
+if(any(!dn2 %in% dn)) warning(toString(dn2[!dn2 %in% dn]),
                              " available in dlf$gof, but not in dlf$parameter.")
-if(any(!dn %in% dn2)) warning(pastec( dn[!dn %in% dn2]),
+if(any(!dn %in% dn2)) warning(toString( dn[!dn %in% dn2]),
                              " available in dlf$parameter, but not in dlf$gof.")
 if(order) dn <- dn2
 output <- matrix(NA, ncol=length(probs), nrow=length(dn) )
@@ -210,7 +240,7 @@ rownames(output) <- dn
 miss <- selection[!selection %in% dn]
 if(length(miss)>0)
   {
-  on.exit(message("Note in distLquantile: specified selection (", pastec(miss),
+  on.exit(message("Note in distLquantile: specified selection (", toString(miss),
                   ") is not available in dlf$gof."), add=TRUE)
   m <- matrix(NA, nrow=length(miss), ncol=ncol(output))
   rownames(m) <- miss # always keep the same order if selection is given
@@ -221,11 +251,20 @@ if(length(miss)>0)
 # and for GPD comparison methods
 if(empirical) output <- rbind(output, quantileMean=NA)
 if(weighted) output <- rbind(output,weighted1=NA, weighted2=NA, weighted3=NA, weightedc=NA)
-if(gpd) output <- rbind(output, q_gpd_evir_pwm=NA, q_gpd_evir_ml=NA,
-    q_gpd_evd=NA, q_gpd_extRemes_MLE=NA, q_gpd_extRemes_GMLE=NA,
-    q_gpd_extRemes_Bayesian=NA, q_gpd_extRemes_Lmoments=NA,
-    q_gpd_fExtremes_pwm=NA, q_gpd_fExtremes_mle=NA, q_gpd_ismev=NA,
-    q_gpd_Renext_r=NA, q_gpd_Renext_f=NA)
+if(gpd) output <- rbind(output, 
+    GPD_LMO_lmomco=NA, 
+    GPD_LMO_extRemes=NA,
+    GPD_PWM_evir=NA, 
+    GPD_PWM_fExtremes=NA, 
+    GPD_MLE_extRemes=NA,
+    GPD_MLE_ismev=NA,
+    GPD_MLE_evd=NA,
+    GPD_MLE_Renext_Renouv=NA, 
+    GPD_MLE_evir=NA, 
+    GPD_MLE_fExtremes=NA,
+    GPD_GML_extRemes=NA, 
+    GPD_MLE_Renext_2par=NA,
+    GPD_BAY_extRemes=NA)
 if(addinfo) output <- rbind(output, n_full=length(dlf$dat_full), n=length(dlf$dat), threshold=dlf$threshold)
 #
 # if input sample size is too small, return NA matrix:
@@ -243,7 +282,7 @@ probs2 <- probs
 if(truncate!=0)
   {
   if(all(probs < truncate) & !quiet) on.exit(message("Note in distLquantile: 'probs' (",
-     pastec(probs),") must contain values that are larger than 'truncate' (", 
+     toString(probs),") must contain values that are larger than 'truncate' (",
           truncate, "). Returning NAs."), add=TRUE)
   probs2 <- (probs-truncate)/(1-truncate) # correct probabilities for smaller sample proportion
   probs2[probs < truncate] <- 0   # avoid negative values
@@ -261,10 +300,16 @@ if(threshold != normalthr)
 #
 # distribution quantiles: ------------------------------------------------------
 # This is the actual work...
-for(d in dn) if(!is.null(dlf$parameter[[d]])) {
-               quantd <- lmomco::qlmomco(f=probs2, para=dlf$parameter[[d]])
-               if(!is.null(quantd)) output[d,] <- quantd
-               }
+for(d in dn) 
+  {
+  if(!is.finite(dlf$gof[d, "RMSE"])) next # maybe this is too restricitive...
+  pard <- dlf$parameter[[d]]              # but it catches gno in skewed samples like
+  if(is.null(pard)) next                  # c(2.4,2.7,2.3,2.5,2.2, 62.4 ,3.8,3.1) 
+  if(inherits(pard, "try-error")) next
+  if(d=="kap") if(pard$ifail!=0) next
+  quantd <- try(lmomco::qlmomco(f=probs2, para=dlf$parameter[[d]]), silent=TRUE)
+  if(!inherits(quantd, "try-error") & !is.null(quantd)) output[d,] <- quantd
+  }
 #
 # Change results for probs below truncate to NA
 if(truncate!=0) output[ , probs < truncate] <- NA
@@ -282,9 +327,10 @@ if(weighted)
       {
       vals <- output[rownames(dlf$gof), col_n]
       if(!any(is.finite(vals))) return(NA)
-      weights <- dlf$gof[is.finite(vals),Weightnr]
-      weights <- weights/sum(weights) # rescale to 1
-      sum(vals[is.finite(vals)] * weights)
+      weights <- dlf$gof[,Weightnr]
+      if(!any(is.finite(weights))) return(NA)
+      weights <- weights/sum(weights, na.rm=TRUE) # rescale to 1
+      sum((vals*weights)[is.finite(vals)&is.finite(weights)])
       })
     }
   output["weighted1",] <- Qweighted("weight1")
@@ -292,7 +338,7 @@ if(weighted)
   output["weighted3",] <- Qweighted("weight3")
   output["weightedc",] <- Qweighted("weightc")
   }
-# q_gpd estimates:
+# q_gpd estimates: -------------------------------------------------------------
 if(gpd)
   {
   # inernal helper function:
@@ -301,23 +347,25 @@ if(gpd)
                         truncate=truncate, threshold=threshold, 
                         quiet=quiet, ttquiet=TRUE)
   #
-  output["q_gpd_evir_pwm",]      <- q_gpd_int("evir", meth="pwm")
-  output["q_gpd_evir_ml",]       <- q_gpd_int("evir", meth="ml")
-  output["q_gpd_evd",]           <- q_gpd_int("evd")
-  output["q_gpd_extRemes_MLE",]  <- q_gpd_int("extRemes", meth="MLE")
-  output["q_gpd_extRemes_GMLE",] <- q_gpd_int("extRemes", meth="GMLE")
-if(!speed) output["q_gpd_extRemes_Bayesian",] <- q_gpd_int("extRemes", meth="Bayesian") # computes a while
-  output["q_gpd_extRemes_Lmoments",] <- q_gpd_int("extRemes", meth="Lmoments")
-  output["q_gpd_fExtremes_pwm",] <- q_gpd_int("fExtremes", meth="pwm")
-  output["q_gpd_fExtremes_mle",] <- q_gpd_int("fExtremes", meth="mle")
-  output["q_gpd_ismev",]         <- q_gpd_int("ismev")
-  output["q_gpd_Renext_r",]      <- q_gpd_int("Renext", meth="r")
-  output["q_gpd_Renext_f",]      <- q_gpd_int("Renext", meth="f")
+  output["GPD_LMO_lmomco",]       <- q_gpd_int("lmomco")
+  output["GPD_LMO_extRemes",]     <- q_gpd_int("extRemes", meth="Lmoments")
+  output["GPD_PWM_evir",]         <- q_gpd_int("evir", meth="pwm")
+  output["GPD_PWM_fExtremes",]    <- q_gpd_int("fExtremes", meth="pwm")
+  output["GPD_MLE_extRemes",]     <- q_gpd_int("extRemes", meth="MLE")
+  output["GPD_MLE_ismev",]        <- q_gpd_int("ismev")
+  output["GPD_MLE_evd",]          <- q_gpd_int("evd")
+  output["GPD_MLE_Renext_Renouv",]<- q_gpd_int("Renext", meth="r")
+  output["GPD_MLE_evir",]         <- q_gpd_int("evir", meth="ml")
+  output["GPD_MLE_fExtremes",]    <- q_gpd_int("fExtremes", meth="mle")
+  output["GPD_GML_extRemes",]     <- q_gpd_int("extRemes", meth="GMLE")
+  output["GPD_MLE_Renext_2par",]  <- q_gpd_int("Renext", meth="f")
+if(!speed)output["GPD_BAY_extRemes",] <- q_gpd_int("extRemes", meth="Bayesian") # computes a while
   }
 #
 dlf$quant <- output
 # Plotting: Quantile lines: ----------------------------------------------------
-if(plot) do.call(distLplot, berryFunctions::owa(c(list(dlf=dlf, qlines=TRUE), list(...)), plotargs))
+if(plot) do.call(distLplot, berryFunctions::owa(c(list(dlf=dlf, qlines=TRUE),
+                                                         list(...)), plotargs))
 # return output: ---------------------------------------------------------------
 if(returnlist) return(dlf) else return(output)
 }
